import { DependencyList, useMemo, useRef } from 'react'

/**
 * 节流刷新
 * @param factory
 * @param deps
 * @param delay
 * @returns
 */
export function useThrottledMemo<T>(factory: () => T, deps: DependencyList, delay?: number): T {
  const time = delay || 5
  const valueRef = useRef<T>()
  const timeRef = useRef<number>(0)

  return useMemo(() => {
    const currentTime = Date.now()
    if (valueRef.current === null || currentTime - timeRef.current >= time) {
      valueRef.current = factory()
      timeRef.current = currentTime
    }

    return valueRef.current
  }, deps) as T
}

/**
 * 防抖刷新
 * @param factory
 * @param deps
 * @param delay
 * @returns
 */
export function useDebouncedMemo<T>(factory: () => T, deps: DependencyList, delay?: number) {
  const time = delay || 5
  const timerRef = useRef<any>()

  return useMemo(() => {
    clearTimeout(timerRef.current)
    timerRef.current = setTimeout(() => {
      factory()
    }, time)

    return () => {
      clearTimeout(timerRef.current)
    }
  }, deps)
}
