import { useCallback, useEffect } from 'react';
import { useStableCallback } from './use-stable-callback';

export type Action = {
  type: string;
  payload?: any;
};

export type Callback = (args?: Action) => void;

export type StackInfo = {
  callbacks: Callback[];
  action?: Action;
};

export const stacks: Record<string, StackInfo | undefined> = {};

export function useNotice(id: string = '', callback?: Callback) {
  const cbk = useStableCallback((...args) => callback?.(...args));

  useEffect(() => {
    if (id && callback) {
      const stack = stacks[id];
      if (stack) {
        stack.callbacks.push(cbk);
        cbk?.(stack.action);
      } else {
        stacks[id] = { callbacks: [cbk] };
      }
      return () => {
        if (stack) {
          stack.callbacks = stack.callbacks.filter((cb) => cb !== cbk);
        }
      };
    }
    return () => {};
  }, []);

  // 分发消息
  const distribute = useCallback((key: string, action: Action) => {
    const stack = stacks[key];
    if (stack) {
      stack.callbacks.forEach((cb) => {
        cb?.(action);
      });
      stack.action = action;
    } else {
      stacks[key] = {
        callbacks: [],
        action,
      };
    }
  }, []);

  const dispatch = useCallback((action: Action, targetId?: string) => {
    if (typeof targetId === 'string') {
      distribute(targetId, action);
    } else {
      Object.keys(stacks).forEach((key) => {
        distribute(key, action);
      });
    }
  }, []);

  return {
    dispatch,
  };
}
