import { DependencyList, useEffect, useMemo, useRef } from 'react';
import { useStableCallback } from './use-stable-callback';
import { debounce } from 'lodash';

export function useResize(
  {
    el,
    onResize = () => {},
  }: {
    el: HTMLElement | null;
    onResize?: (entry: ResizeObserverEntry) => void;
  },
  deps?: DependencyList,
) {
  const prevRect = useRef<ResizeObserverEntry['contentRect']>();
  const onResizeFn = useStableCallback(onResize);

  const obs = useMemo(() => {
    const fn: ResizeObserverCallback = (entries) => {
      entries.forEach((entry) => {
        if (
          prevRect.current?.width !== entry.contentRect.width ||
          prevRect.current?.height !== entry.contentRect.height
        ) {
          onResizeFn?.(entry);
        }
        prevRect.current = entry.contentRect;
      });
    };
    return new ResizeObserver(debounce(fn, 5));
  }, []);

  useEffect(() => {
    if (el) {
      // 开始监听
      obs?.observe(el);
      return () => {
        obs?.unobserve(el);
        obs?.disconnect();
      };
    }
    return undefined;
  }, [obs, el, ...(deps || [])]);
}
