import { RefObject, useEffect, useLayoutEffect, useRef, useState } from 'react';
import { useResize } from './use-resize';
import { Size, useScale } from './use-scale';

type BasicType = HTMLElement | null;

type TargetType = BasicType | RefObject<HTMLDivElement>;

function getElement(target: TargetType) {
  if ('current' in (target as RefObject<HTMLDivElement>)) {
    return (target as RefObject<HTMLDivElement>).current;
  }
  return target as BasicType;
}

export function useTargetScale(keepSize = false) {
  const container = useRef<HTMLDivElement>(null);
  const target = useRef<HTMLDivElement>(null);

  const [containerSize, setContainerSize] = useState<Size>({
    width: 0,
    height: 0,
  });
  const [targetSize, setTargetSize] = useState<Size>();

  const [containerEl, setContainerEl] = useState<BasicType>(null);
  const [targetEl, setTargetEl] = useState<BasicType>(null);

  useEffect(() => {
    setContainerEl(getElement(container));
  }, [container]);

  useEffect(() => {
    setTargetEl(getElement(target));
  }, [target]);

  useResize({
    el: containerEl,
    onResize({ contentRect: { width, height } }) {
      setContainerSize({ width, height });
    },
  });

  useLayoutEffect(() => {
    if (targetEl) {
      const { offsetWidth: width, offsetHeight: height } = targetEl;
      setTargetSize({ width, height });
    }
  }, [targetEl]);

  const scale = useScale(containerSize, targetSize, keepSize);
  return {
    scale,
    container,
    target,
  };
}
