import { useEffect, useRef } from 'react';

interface IProps {
  onPushState?: (param?: any) => void;
  onReplaceState?: (param?: any) => void;
}

const _wr = (type: 'pushState' | 'replaceState') => {
  const origin = window.history[type];
  return function () {
    // @ts-ignore
    const newEvent = origin.apply(this, arguments);
    const e: any = new Event(type);
    e.arguments = arguments;
    window.dispatchEvent(e);
    return newEvent;
  };
};

export function useVisitor(param: IProps, age: any[]) {
  const timer = useRef<number | any>();
  const onPrevEvent = useRef<IProps>(param);

  const onEvent = (event: 'onPushState' | 'onReplaceState', e?: any) => {
    if (timer.current) {
      clearTimeout(timer.current);
    }
    timer.current = setTimeout(() => {
      let data = undefined;
      if (e && e?.arguments?.[0]) {
        data = e?.arguments[0];
      }
      onPrevEvent.current[event]?.(data);
    }, 5);
  };

  const onPushStatePrev = (e?: any) => {
    onEvent('onPushState', e);
  };

  const onReplaceStatePrev = (e?: any) => {
    onEvent('onReplaceState', e);
  };

  useEffect(() => {
    onPrevEvent.current = param;
  }, [age]);

  const unregister = () => {
    window.removeEventListener('popstate', onPushStatePrev);
    window.removeEventListener('pushState', onPushStatePrev);
    window.removeEventListener('replaceState', onReplaceStatePrev);
  };

  const register = () => {
    window.addEventListener('popstate', onPushStatePrev);
    window.addEventListener('pushState', onPushStatePrev);
    window.addEventListener('replaceState', onReplaceStatePrev);
  };

  useEffect(() => {
    window.history.pushState = _wr('pushState');
    window.history.replaceState = _wr('replaceState');

    register();
    return () => {
      unregister();
    };
  }, []);
}
