import { useMemo, useRef, useState } from 'react';
import { useResize } from './use-resize';
import { Size, useScale } from './use-scale';

export function useZoom(initialSize: Size, keepSize = false): any {
  const container = useRef<HTMLDivElement>(null);
  const target = useRef<HTMLDivElement>(null);

  const [containerSize, setContainerSize] = useState<Size>({
    width: 0,
    height: 0,
  });

  const scale = useScale(containerSize, initialSize, keepSize);

  useResize({
    el: container.current,
    onResize({ contentRect: { width, height } }) {
      setContainerSize({ width, height });
    },
  });

  const computedSize: Size = useMemo(() => {
    const { width, height } = initialSize;
    return {
      width: width * scale,
      height: height * scale,
    };
  }, [scale, initialSize]);

  return {
    initialSize,
    computedSize,
    scale,
    container,
    target,
  };
}
